Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Runtime.InteropServices

Namespace VSCom.J1939Api

    Class VSCAN_J1939

        Public Const errHandleInvalid As String = "Handle is invalid (no Init done?)"

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure VSCAN_J1939_VERSION

            Public Major As Byte

            Public Minor As Byte

            Public SubMinor As Byte
        End Structure

        Public Structure VSCAN_J1939_MSG

            Public PGN As UInt32

            Public Data() As Byte

            Public Size As UInt16

            Public Dst As Byte

            Public Src As Byte

            Public Priority As Byte

            Public Status As Byte
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure VSCAN_J1939_MSG_NATIVE

            Public PGN As UInt32

            Public Data As IntPtr

            Public Size As UInt16

            Public Dst As Byte

            Public Src As Byte

            Public Priority As Byte

            Public Status As Byte
        End Structure

        <StructLayout(LayoutKind.Sequential)> _
        Public Structure VSCAN_J1939_NAME

            Public Id As UInt32

            Public ManCode As UInt16

            Public EcuInst As Byte

            Public FuncInst As Byte

            Public Func As Byte

            Public VehicleSys As Byte

            Public VehicleSysInst As Byte

            Public IndustryGroup As Byte

            Public ArbAddrCapable As Byte
        End Structure

        Public Delegate Function VSCAN_J1939_ADDR_CALLBACK() As Byte

        Public Delegate Sub VSCAN_J1939_PGN_CALLBACK(ByVal Msg As IntPtr)

        Public Const VSCAN_J1939_ERR_OK As Int32 = 0

        Public Const VSCAN_J1939_ERR_ERR As Int32 = -1

        Public Const VSCAN_J1939_ERR_INVALID_HANDLE As Int32 = -2

        Public Const VSCAN_J1939_ERR_INVALID_PARAMETER As Int32 = -3

        Public Const VSCAN_J1939_ERR_MEMORY As Int32 = -4

        Public Const VSCAN_J1939_ERR_SUBAPI As Int32 = -5

        Public Const VSCAN_J1939_ERR_ADDR_UNAVAILABLE As Int32 = -6

        Public Const VSCAN_J1939_ERR_ADDR_CLAIM As Int32 = -7

        Public Const VSCAN_J1939_ERR_DATA_TO_SMALL As Int32 = -8

        Public Const VSCAN_J1939_STATUS_SENDING As Byte = 1

        Public Const VSCAN_J1939_STATUS_SENT As Byte = 2

        Public Const VSCAN_J1939_STATUS_ERROR As Byte = 3

        Public Const VSCAN_J1939_STATUS_FILL_IN As Byte = 255

        Public Const VSCAN_J1939_PGN_REQUEST As Int32 = 59904

        Public Const VSCAN_J1939_PGN_ACK As Int32 = 59392

        Public Const VSCAN_J1939_ADDR_BROADCAST As Byte = 255

        Private Declare Function VSCAN_J1939_Init Lib "vs_can_j1939" Alias "VSCAN_J1939_Init" (ByVal Handle As Int32, ByVal Name As VSCAN_J1939_NAME, ByVal Callback As VSCAN_J1939_ADDR_CALLBACK) As Int32

        Private Declare Function VSCAN_J1939_Free Lib "vs_can_j1939" Alias "VSCAN_J1939_Free" (ByVal Handle As Int32) As Int32

        Private Declare Function VSCAN_J1939_Write Lib "vs_can_j1939" Alias "VSCAN_J1939_Write" (ByVal Handle As Int32, ByVal Buf() As VSCAN_J1939_MSG_NATIVE, ByVal Size As UInt32, ByRef Written As UInt32) As Int32

        Private Declare Function VSCAN_J1939_Request Lib "vs_can_j1939" Alias "VSCAN_J1939_Request" (ByVal Handle As Int32, ByVal Pgn As UInt32, ByVal Addr As Byte) As Int32

        Private Declare Function VSCAN_J1939_Read Lib "vs_can_j1939" Alias "VSCAN_J1939_Read" (ByVal Handle As Int32, ByVal Buf As IntPtr, ByVal Size As UInt32, ByRef Read As UInt32) As Int32

        Private Declare Function VSCAN_J1939_SetPgnCallback Lib "vs_can_j1939" Alias "VSCAN_J1939_SetPgnCallback" (ByVal Handle As Int32, ByVal Pgn As UInt32, ByVal Callback As VSCAN_J1939_PGN_CALLBACK) As Int32

        Private Declare Function VSCAN_J1939_Read_BufferCount Lib "vs_can_j1939" Alias "VSCAN_J1939_Read_BufferCount" (ByVal Handle As Int32) As UInt32

        Private Declare Sub VSCAN_J1939_Read_BufferClear Lib "vs_can_j1939" Alias "VSCAN_J1939_Read_BufferClear" (ByVal Handle As Int32)

        Private Declare Function VSCAN_J1939_ApiVersion Lib "vs_can_j1939" Alias "VSCAN_J1939_ApiVersion" () As VSCAN_J1939_VERSION

        Private handle As Int32 = 0

        Public Sub Init(ByVal Handle As Int32, ByVal Name As VSCAN_J1939_NAME, ByVal Callback As VSCAN_J1939_ADDR_CALLBACK)
            If (Handle <= 0) Then
                Throw New VSCANException("Handle is invalid")
            End If

            Me.handle = Handle
            Dim rc As Int32 = VSCAN_J1939.VSCAN_J1939_Init(Handle, Name, Callback)
            If (rc <> VSCAN_J1939_ERR_OK) Then
                Throw New VSCANException(("Init failed " + rc))
            End If

        End Sub

        Public Sub Free()
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            Dim rc As Int32 = VSCAN_J1939.VSCAN_J1939_Free(Me.handle)
            If (rc <> VSCAN_J1939_ERR_OK) Then
                Throw New VSCANException(("Free failed " + rc))
            End If

            Me.handle = 0
        End Sub

        Public Sub Write(ByVal Buf() As VSCAN_J1939_MSG, ByVal Size As UInt32, ByRef Written As UInt32)
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            Dim NBuf() As VSCAN_J1939_MSG_NATIVE = New VSCAN_J1939_MSG_NATIVE((Size) - 1) {}
            Dim i As Integer = 0
            Do While (i < Size)
                NBuf(i).PGN = Buf(i).PGN
                NBuf(i).Src = Buf(i).Src
                NBuf(i).Dst = Buf(i).Dst
                NBuf(i).Priority = Buf(i).Priority
                NBuf(i).Status = Buf(i).Status
                If (Buf(i).Size = 0) Then
                    NBuf(i).Size = CType(Buf(i).Data.Length, UInt16)
                Else
                    NBuf(i).Size = Buf(i).Size
                End If

                NBuf(i).Data = Marshal.AllocHGlobal((Marshal.SizeOf(GetType(System.Byte)) * NBuf(i).Size))
                Marshal.Copy(Buf(i).Data, 0, NBuf(i).Data, NBuf(i).Size)
                i = (i + 1)
            Loop

            Dim rc As Int32 = VSCAN_J1939.VSCAN_J1939_Write(Me.handle, NBuf, Size, Written)
            i = 0
            Do While (i < Size)
                Marshal.FreeHGlobal(NBuf(i).Data)
                i = (i + 1)
            Loop

            If (rc <> VSCAN_J1939_ERR_OK) Then
                Throw New VSCANException(("Write failed " + rc))
            End If

        End Sub

        Public Sub Request(ByVal Pgn As UInt32, ByVal Addr As Byte)
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            Dim rc As Int32 = VSCAN_J1939.VSCAN_J1939_Request(Me.handle, Pgn, Addr)
            If (rc <> VSCAN_J1939_ERR_OK) Then
                Throw New VSCANException(("Request failed " + rc))
            End If

        End Sub

        Public Sub Read(ByRef Buf() As VSCAN_J1939_MSG, ByVal Size As UInt32, ByRef Read As UInt32)
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            Dim NBuf() As VSCAN_J1939_MSG_NATIVE = New VSCAN_J1939_MSG_NATIVE((Size) - 1) {}
            Dim pBuf As IntPtr = Marshal.AllocHGlobal((Marshal.SizeOf(NBuf(0)) * CType(Size, Integer)))
            Marshal.StructureToPtr(NBuf(0), pBuf, False)
            Read = 0
            Dim rc As Int32 = VSCAN_J1939.VSCAN_J1939_Read(Me.handle, pBuf, Size, Read)
            Dim pCurrent As IntPtr = pBuf
            Dim i As Integer = 0
            Do While (i < Read)
                NBuf(i) = CType(Marshal.PtrToStructure(pCurrent, GetType(VSCAN_J1939_MSG_NATIVE)), VSCAN_J1939_MSG_NATIVE)
                pCurrent = CType((CType(pCurrent, Integer) + Marshal.SizeOf(NBuf(0))), IntPtr)
                i = (i + 1)
            Loop

            Marshal.FreeHGlobal(pBuf)
            If (rc <> VSCAN_J1939_ERR_OK) Then
                Throw New VSCANException(("Read failed " + rc))
            End If

            i = 0
            Do While (i < Size)
                Buf(i).PGN = NBuf(i).PGN
                Buf(i).Src = NBuf(i).Src
                Buf(i).Dst = NBuf(i).Dst
                Buf(i).Priority = NBuf(i).Priority
                Buf(i).Status = NBuf(i).Status
                Buf(i).Size = NBuf(i).Size
                Buf(i).Data = New Byte((Buf(i).Size) - 1) {}
                Marshal.Copy(CType(NBuf(i).Data, IntPtr), Buf(i).Data, 0, NBuf(i).Size)
                i = (i + 1)
            Loop

        End Sub

        Public Sub SetPgnCallback(ByVal Pgn As UInt32, ByVal Callback As VSCAN_J1939_PGN_CALLBACK)
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            Dim rc As Int32 = VSCAN_J1939.VSCAN_J1939_SetPgnCallback(Me.handle, Pgn, Callback)
            If (rc <> VSCAN_J1939_ERR_OK) Then
                Throw New VSCANException(("SetPgnCallback failed " + rc))
            End If

        End Sub

        Public Function Read_BufferCount() As UInt32
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            Return VSCAN_J1939.VSCAN_J1939_Read_BufferCount(Me.handle)
        End Function

        Public Sub Read_BufferClear()
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            VSCAN_J1939.VSCAN_J1939_Read_BufferClear(Me.handle)
        End Sub

        Public Function ApiVersion() As VSCAN_J1939_VERSION
            If (Me.handle <= 0) Then
                Throw New VSCANException(errHandleInvalid)
            End If

            Return VSCAN_J1939.VSCAN_J1939_ApiVersion
        End Function
    End Class
End Namespace