var ref = require('ref');
var ffi = require('ffi');
var Struct = require('ref-struct');

var vsCanHandle = ref.types.int;
var vsCanStatus = ref.types.int;

const VSCAN_IOCTL_SET_DEBUG = 1;
const VSCAN_IOCTL_SET_SPEED = 3;
const VSCAN_IOCTL_SET_ACC_CODE_MASK = 6;
const VSCAN_IOCTL_SET_DEBUG_MODE = 8;
var temp = ref.alloc('void*');
temp.writeInt32LE(-101);
const VSCAN_DEBUG_HIGH = temp.deref();
var temp = ref.alloc('void*');
temp.writeInt32LE(1);
const VSCAN_DEBUG_MODE_CONSOLE = temp.deref();
var temp = ref.alloc('void*');
temp.writeInt32LE(5);
const VSCAN_SPEED_250K = temp.deref();

var vsCanCodeMask = Struct({
  'Code' : 'uint32',
  'Mask' : 'uint32'
});

var vsJ1939Status = ref.types.int;
var vsJ1939Addr = ref.types.uint8;
var vsJ1939Pgn = ref.types.uint32;
var vsJ1939Name = Struct({
  'Id' : 'uint32',
  'ManCode' : 'uint16',
  'EcuInst' : 'uint8',
  'FuncInst' : 'uint8',
  'Func' : 'uint8',
  'VehicleSys' : 'uint8',
  'VehicleSysInst' : 'uint8',
  'IndustryGroup' : 'uint8',
  'ArbAddrCapable' : 'uint8',
});
var vsJ1939Msg = Struct({
  'Pgn' : 'uint32',
  'Data' : ref.refType(ref.refType("void")),
  'Size' : 'uint16',
  'Dst' : 'uint8',
  'Src' : 'uint8',
  'Priority' : 'uint8',
  'Status' : 'uint8',
});
var vsJ1939MsgPtr = ref.refType(vsJ1939Msg);
var vsJ1939Version = Struct({
  'Major' : 'uint8',
  'Minor' : 'uint8',
  'SubMinor' : 'uint8',
});
var vsJ1939AddrCallback = ffi.Function(vsJ1939Addr, []);
var vsJ1939PgnCallback = ffi.Function('void', [vsJ1939MsgPtr]);

if (process.arch == 'x64')
{
  libFileJ1939 = 'libvs_can_j1939_x86-64.so';
  libFileVSCAN = 'libvs_can_api_x86-64.so';
}
else
{
  libFileJ1939 = 'libvs_can_j1939.so';
  libFileVSCAN = 'libvs_can_api.so';
}

// loads all symbols from the shared object for the libvs_can_j1939.so
ffi.DynamicLibrary(libFileVSCAN, ffi.DynamicLibrary.FLAGS.RTLD_NOW |
                                 ffi.DynamicLibrary.FLAGS.RTLD_GLOBAL);

var vscan = ffi.Library(libFileVSCAN, {
  'VSCAN_Open': [vsCanHandle, ['CString', 'uint32']],
  'VSCAN_Close': ['void', [vsCanHandle]],
  'VSCAN_Ioctl': [vsCanStatus, [vsCanHandle, 'uint32', 'void*']],
});

var j1939 = ffi.Library(libFileJ1939, {
  'VSCAN_J1939_Init': [vsJ1939Status, [vsCanHandle, vsJ1939Name, vsJ1939AddrCallback]],
  'VSCAN_J1939_Free': [vsJ1939Status, [vsCanHandle]],
  'VSCAN_J1939_Write': [vsJ1939Status, [vsCanHandle, vsJ1939MsgPtr, 'uint32', 'uint32*']],
  'VSCAN_J1939_Read': [vsJ1939Status, [vsCanHandle, vsJ1939MsgPtr, 'uint32', 'uint32*']],
  'VSCAN_J1939_Request': [vsJ1939Status, [vsCanHandle, vsJ1939Pgn, vsJ1939Addr]],
  'VSCAN_J1939_SetPgnCallback': [vsJ1939Status, [vsCanHandle, vsJ1939Pgn, vsJ1939PgnCallback]],
  'VSCAN_J1939_ApiVersion': [vsJ1939Version, []],
  'VSCAN_J1939_Read_BufferCount': ['uint32', [vsCanHandle]],
});

// exports for our node.js module
exports.vsCanHandle = vsCanHandle;
exports.vsCanStatus = vsCanStatus;
exports.VSCAN_IOCTL_SET_DEBUG = VSCAN_IOCTL_SET_DEBUG;
exports.VSCAN_IOCTL_SET_SPEED = VSCAN_IOCTL_SET_SPEED;
exports.VSCAN_IOCTL_SET_ACC_CODE_MASK = VSCAN_IOCTL_SET_ACC_CODE_MASK;
exports.VSCAN_IOCTL_SET_DEBUG_MODE = VSCAN_IOCTL_SET_DEBUG_MODE;
exports.VSCAN_DEBUG_HIGH = VSCAN_DEBUG_HIGH;
exports.VSCAN_DEBUG_MODE_CONSOLE = VSCAN_DEBUG_MODE_CONSOLE;
exports.VSCAN_SPEED_250K = VSCAN_SPEED_250K;
exports.vsCanCodeMask = vsCanCodeMask;
exports.VSCAN_Open = vscan.VSCAN_Open;
exports.VSCAN_Ioctl = vscan.VSCAN_Ioctl;
exports.VSCAN_Close = vscan.VSCAN_Close;
exports.vsJ1939Status = vsJ1939Status;
exports.vsJ1939Addr = vsJ1939Addr;
exports.vsJ1939Pgn = vsJ1939Pgn;
exports.vsJ1939Name = vsJ1939Name;
exports.vsJ1939Msg = vsJ1939Msg;
exports.vsJ1939MsgPtr = vsJ1939MsgPtr;
exports.vsJ1939Version = vsJ1939Version;
exports.vsJ1939AddrCallback = vsJ1939AddrCallback;
exports.vsJ1939PgnCallback = vsJ1939PgnCallback;
exports.VSCAN_J1939_Init = j1939.VSCAN_J1939_Init;
exports.VSCAN_J1939_Free = j1939.VSCAN_J1939_Free;
exports.VSCAN_J1939_Write = j1939.VSCAN_J1939_Write;
exports.VSCAN_J1939_Read = j1939.VSCAN_J1939_Read;
exports.VSCAN_J1939_Request = j1939.VSCAN_J1939_Request;
exports.VSCAN_J1939_SetPgnCallback = j1939.VSCAN_J1939_SetPgnCallback;
exports.VSCAN_J1939_ApiVersion = j1939.VSCAN_J1939_ApiVersion;
exports.VSCAN_J1939_Read_BufferCount = j1939.VSCAN_J1939_Read_BufferCount;
